/* GETNETMASK.C
 * Paul A. Whittaker <Paul.Whittaker@iname.com> 2002
 *
 * A small helper program which performs a lookup function not yet implemented
 * by GLibC, namely finding and reporting the longest matching netmask in
 * /etc/netmasks for the IP address argument provided.  If no match can be
 * found, the program attempts to guess the netmask, using the same rules that
 * the kernel uses when no netmask is provided (ie. first octet in range 0-127
 * => A-class; 128-191 => B-class; 192-223 => C-class; 224-255 invalid).
 *
 * Result codes:
 *     -1 = syntax error
 *	0 = matched
 *	1 = guessed
 *	2 = failed
*/

#include <netinet/in.h>
#include <stdio.h>

#define ip(a,b,c,d)	((a<<24)|(b<<16)|(c<<8)|d)

int main (int argc, char *argv[]) {

	uint8_t a, b, c, d, e, f, g, h, len, guessed, bestlen = 0;
	uint32_t tgt_ip, netnum, mask, maskcopy, bestmask = 0;
	FILE *db;

	if ((argc != 2) || (sscanf (argv[1], "%hhu.%hhu.%hhu.%hhu", &a, &b, &c,
				&d) != 4)) {
		fprintf (stderr, "Usage: %s IP-address\n", argv[0]);
		return (-1);
	}
	tgt_ip = ip (a, b, c, d);
	db = fopen ("/etc/netmasks", "r");
	if (db != NULL) {
		while (!feof (db)) {
			if (fscanf (db, "%hhu.%hhu.%hhu.%hhu \
						%hhu.%hhu.%hhu.%hhu\n", &a, &b,
						&c, &d, &e, &f, &g, &h) != 8) {
				/* quietly ignore bogus entries. */
				continue;
			}
			netnum = ip (a, b, c, d);
			mask = ip (e, f, g, h);
			if ((tgt_ip & mask) == netnum) {
				len = 0;
				maskcopy = mask;
				while (maskcopy) {
					maskcopy <<= 1;
					len++;
				}
				if (len > bestlen) {
					bestlen = len;
					bestmask = mask;
				}
			}
		}
		fclose (db);
	}
	guessed = (bestmask == 0);
	if (guessed) {
		bestmask = IN_CLASSA (tgt_ip) ? IN_CLASSA_NET : IN_CLASSB
			(tgt_ip) ? IN_CLASSB_NET : IN_CLASSC (tgt_ip) ?
			IN_CLASSC_NET : 0;
	}
	if (bestmask) {
		fprintf (stdout, "%hhu.%hhu.%hhu.%hhu\n", (bestmask &
				0xff000000) >> 24, (bestmask & 0x00ff0000) >>
				16, (bestmask & 0x0000ff00) >> 8, bestmask &
				0x000000ff);
		return (guessed);
	}
	fputs ("Unable to determine netmask.\n", stderr);
	return (2);
}
